﻿local easyLootLevels = {
	"Poor",
	"Common",
	"Uncommon",
	"Rare",
	"Epic",
	"Legendary",
	"Artifact"
}

local panelName

local function eventHandler(self, event, ...)
  if (event=="LOOT_OPENED") then
  	local arg1 = ...
  	if(arg1 == 0 and EasyLootSettings.enabled) then
		EasyLoot_HandleLoot()
	elseif(not EasyLootSettings.enabled) then
			--DEFAULT_CHAT_FRAME:AddMessage("|cffffff00EasyLoot disabled")
  	end
  elseif (event=="START_LOOT_ROLL") then
  	local id, timer = ...
  	EasyLoot_HandleRoll(id)
  elseif (event=="CONFIRM_LOOT_ROLL") then
  	local id, rolltype = ...
  	EasyLoot_HandleConfirmation(id, rolltype)
  elseif (event=="CHAT_MSG_LOOT") then
  	local message, sender, language, channelString, target, flags, _, channelNumber, channelName, _, _ = ...
	EasyLoot_HandleIncomingLoot(message)
  elseif (event=="VARIABLES_LOADED") then
  end
end

function EasyLoot_OnLoad()
	local version = GetAddOnMetadata("EasyLoot", "Version")
	EasyLoot_SetVariables()

	this:SetScript("OnEvent", eventHandler)
	this:RegisterEvent("VARIABLES_LOADED")

	SlashCmdList["EASYLOOT"] = function(msg)
		EasyLoot_SlashCommand(msg);
	end;
	SLASH_EASYLOOT1 = "/el"

	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00EasyLoot v"..version.." loaded")
	end
	UIErrorsFrame:AddMessage("EasyLoot v"..version.." AddOn loaded", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME)
	this:RegisterEvent("LOOT_OPENED")
	this:RegisterEvent("START_LOOT_ROLL")
 	this:RegisterEvent("CONFIRM_LOOT_ROLL")
	this:RegisterEvent("CHAT_MSG_LOOT")
end

function EasyLoot_SetVariables()
	if(not EasyLootSettings) then
		EasyLootSettings = {}
		EasyLootSettings.rarity = 1
		EasyLootSettings.enabled = true
		EasyLootSettings.freeforall = true
		EasyLootSettings.roundrobin = true
		EasyLootSettings.master = true
		EasyLootSettings.group = true
		EasyLootSettings.needbeforegreed = true
	end
	if(not EasyLootLootList) then
		EasyLootLootList = {}
		EasyLootLootList.ignore = {}
		EasyLootLootList.autoloot = {}
		EasyLootLootList.need = {}
	end
end

function EasyLoot_SlashCommand(msg)
  if(msg) then
    local command = strlower(msg)
	if (command == "options") then
    	InterfaceOptionsFrame_OpenToCategory(panelName);
    end
  end
end

function EasyLoot_HandleLoot()
	local numItems = GetNumLootItems()
	--DEFAULT_CHAT_FRAME:AddMessage("|cffffff00EasyLoot number of items:" .. numItems)
	for i=1,numItems do
		lootIcon, lootName, lootQuantity, rarity, locked = GetLootSlotInfo(i)
		if((rarity >= EasyLootSettings.rarity or not LootSlotIsItem(i) or EasyLoot_InTable(EasyLootLootList.autoloot, lootName)) and not(EasyLoot_InTable(EasyLootLootList.ignore, lootName)) and not locked and EasyLootSettings[GetLootMethod()]) then
			LootSlot(i)
		else
			--DEFAULT_CHAT_FRAME:AddMessage("|cffffff00EasyLoot ignoring item:" .. lootName.." rarity: "..rarity)
		end
	end
end

function EasyLoot_HandleRoll(id)
	local texture, name, count, quality, bindOnPickUp = GetLootRollItemInfo(id)
  	--DEFAULT_CHAT_FRAME:AddMessage("|cffffff00EasyLoot item up for rolling: "..name.." Link: "..GetLootRollItemLink(id))
  	if(EasyLoot_InTable(EasyLootLootList.need, name)) then
  		RollOnLoot(id, 1)
  	end
end

function EasyLoot_HandleConfirmation(id, rolltype)
	local texture, name, count, quality, bindOnPickUp = GetLootRollItemInfo(id)
	if(rolltype == 1 and EasyLoot_InTable(EasyLootLootList.need, name)) then
		ConfirmLootRoll(id, rolltype)
	end
end

function EasyLoot_HandleIncomingLoot(message)
	for i=1,getn(EasyLootLootList.need),1  do
		if(strmatch(message, "You .*"..EasyLootLootList.need[i]..".*")) then
			--DEFAULT_CHAT_FRAME:AddMessage("|cffffff00EasyLoot Match")
			tremove(EasyLootLootList.need, i)
			EasyLoot_NeedScrollBar_Update()
			return
		end
	end
end

function EasyLoot_ConfigLoadOrCancel()
	EasyLootConfigFrameEnabled:SetChecked(EasyLootSettings.enabled)
	EasyLootConfigFrameFreeForAll:SetChecked(EasyLootSettings.freeforall)
	EasyLootConfigFrameRoundRobin:SetChecked(EasyLootSettings.roundrobin)
	EasyLootConfigFrameMaster:SetChecked(EasyLootSettings.master)
	EasyLootConfigFrameGroup:SetChecked(EasyLootSettings.group)
	EasyLootConfigFrameNeedBeforeGreed:SetChecked(EasyLootSettings.needbeforegreed)
end

function EasyLootConfigFrame_OnLoad(panel)
	panelName = "EasyLoot " .. GetAddOnMetadata("EasyLoot", "Version")
    panel.name = panelName
    panel.okay = function(self)
    	EasyLootConfig_Ok()
    end
    panel.cancel = function(self)
    	EasyLoot_ConfigLoadOrCancel()
    end
    InterfaceOptions_AddCategory(panel);
end

function EasyLootConfig_Ok()
	EasyLootSettings.enabled=EasyLootConfigFrameEnabled:GetChecked()
	EasyLootSettings.freeforall=EasyLootConfigFrameFreeForAll:GetChecked()
	EasyLootSettings.roundrobin=EasyLootConfigFrameRoundRobin:GetChecked()
	EasyLootSettings.master=EasyLootConfigFrameMaster:GetChecked()
	EasyLootSettings.group=EasyLootConfigFrameGroup:GetChecked()
	EasyLootSettings.needbeforegreed = EasyLootConfigFrameNeedBeforeGreed:GetChecked()
end

function EasyLoot_RarityDropdown_OnClick()
  local i = this:GetID()
  EasyLootSettings.rarity = i-1
  UIDropDownMenu_SetSelectedID(EasyLootConfigFrameComboRarity, i)
end

function EasyLoot_RarityDropdown_OnShow()
	UIDropDownMenu_Initialize(EasyLootConfigFrameComboRarity, EasyLoot_RarityDropdown_Initialize)
	UIDropDownMenu_SetSelectedID(EasyLootConfigFrameComboRarity, EasyLootSettings.rarity+1)
end

function EasyLoot_RarityDropdown_Initialize()
	local i;
	if(getn(easyLootLevels) > 0) then
		for i = 1, getn(easyLootLevels), 1 do
			local redComponent, greenComponent, blueComponent, hexColor = GetItemQualityColor(i-1)
			info = {
				text = hexColor..easyLootLevels[i];
				func = EasyLoot_RarityDropdown_OnClick;
			};
			UIDropDownMenu_AddButton(info);
			i = i + 1;
		end
	end
end

function EasyLoot_InTable(t, val)
	for _,v in pairs(t) do
  		if v == val then
		    return true
  		end
	end
	return false
end

function EasyLoot_AutoLootScrollBar_Update()
	local line
	local lineplusoffset
	FauxScrollFrame_Update(EasyLootFilterAutoLootScrollFrame, getn(EasyLootLootList.autoloot), 5, 16)
	for line=1,5 do
		lineplusoffset = line + FauxScrollFrame_GetOffset(EasyLootFilterAutoLootScrollFrame)
		if lineplusoffset <= getn(EasyLootLootList.autoloot) then
			if(strlen(EasyLootLootList.autoloot[lineplusoffset]) > 12) then
				text = strsub(EasyLootLootList.autoloot[lineplusoffset],1,12).."..."
			else
				text = EasyLootLootList.autoloot[lineplusoffset]
			end
			getglobal("EasyLootAutoLootEntry"..line):SetText(text)
			getglobal("EasyLootAutoLootEntry"..line):Show()
		else
			getglobal("EasyLootAutoLootEntry"..line):Hide()
		end
	end
end

function EasyLoot_IgnoreScrollBar_Update()
	local line
	local lineplusoffset
	FauxScrollFrame_Update(EasyLootFilterIgnoreScrollFrame, getn(EasyLootLootList.ignore), 5, 16)
	for line=1,5 do
		lineplusoffset = line + FauxScrollFrame_GetOffset(EasyLootFilterIgnoreScrollFrame)
		if lineplusoffset <= getn(EasyLootLootList.ignore) then
			if(strlen(EasyLootLootList.ignore[lineplusoffset]) > 12) then
				text = strsub(EasyLootLootList.ignore[lineplusoffset],1,12).."..."
			else
				text = EasyLootLootList.autoloot[lineplusoffset]
			end
			getglobal("EasyLootIgnoreEntry"..line):SetText(text)
			getglobal("EasyLootIgnoreEntry"..line):Show()
		else
			getglobal("EasyLootIgnoreEntry"..line):Hide()
		end
	end
end

function EasyLoot_NeedScrollBar_Update()
	local line
	local lineplusoffset
	FauxScrollFrame_Update(EasyLootFilterNeedScrollFrame, getn(EasyLootLootList.need), 5, 16)
	for line=1,5 do
		lineplusoffset = line + FauxScrollFrame_GetOffset(EasyLootFilterNeedScrollFrame)
		if lineplusoffset <= getn(EasyLootLootList.need) then
			if(strlen(EasyLootLootList.need[lineplusoffset]) > 12) then
				text = strsub(EasyLootLootList.need[lineplusoffset],1,12).."..."
			else
				text = EasyLootLootList.need[lineplusoffset]
			end
			getglobal("EasyLootNeedEntry"..line):SetText(text)
			getglobal("EasyLootNeedEntry"..line):Show()
		else
			getglobal("EasyLootNeedEntry"..line):Hide()
		end
	end
end

function EasyLoot_ListButtonClicked(button)
	local text = button:GetText()
	local name = button:GetName()
	if(strmatch(name, "^EasyLootAutoLootEntry")) then
		local o = FauxScrollFrame_GetOffset(EasyLootFilterAutoLootScrollFrame)
		local line = tonumber(strmatch(name, "%d"))
		tremove(EasyLootLootList.autoloot, o+line)
		EasyLoot_AutoLootScrollBar_Update()
	elseif(strmatch(name, "^EasyLootIgnoreEntry")) then
		local o = FauxScrollFrame_GetOffset(EasyLootFilterIgnoreScrollFrame)
		local line = tonumber(strmatch(name, "%d"))
		tremove(EasyLootLootList.ignore, o+line)
		EasyLoot_IgnoreScrollBar_Update()
	elseif(strmatch(name, "^EasyLootNeedEntry")) then
		local o = FauxScrollFrame_GetOffset(EasyLootFilterNeedScrollFrame)
		local line = tonumber(strmatch(name, "%d"))
		tremove(EasyLootLootList.need, o+line)
		EasyLoot_NeedScrollBar_Update()
	end
end

function EasyLoot_SetHook()
	Old_ChatEdit_InsertLink = ChatEdit_InsertLink
	ChatEdit_InsertLink = function(text)
		ChatEdit_InsertLink = Old_ChatEdit_InsertLink
		return EasyLoot_InsertLink(text) or Old_ChatEdit_InsertLink(text)
	end
end

function EasyLoot_InsertLink(text)
	--DEFAULT_CHAT_FRAME:AddMessage(text)
	local _, _, _, _, _, _, _, _, _, _, _, _, _, Name = string.find(text, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
	EasyLootFilterItem:SetText(Name)
	return true
end

function EasyLoot_AddAutoLoot()
	local itemName = EasyLootFilterItem:GetText()
	if(itemName and not (itemName=="")) then
		EasyLootLootList.autoloot[getn(EasyLootLootList.autoloot) + 1] = itemName
		EasyLoot_AutoLootScrollBar_Update()
		EasyLootFilterItem:SetText("")
	end
	EasyLootFilterItem:ClearFocus()
end

function EasyLoot_AddIgnore()
	local itemName = EasyLootFilterItem:GetText()
	if(itemName and not (itemName=="")) then
		EasyLootLootList.ignore[getn(EasyLootLootList.ignore) + 1] = itemName
		EasyLoot_IgnoreScrollBar_Update()
		EasyLootFilterItem:SetText("")
	end
	EasyLootFilterItem:ClearFocus()
end

function EasyLoot_AddNeed()
	local itemName = EasyLootFilterItem:GetText()
	if(itemName and not (itemName=="")) then
		EasyLootLootList.need[getn(EasyLootLootList.need) + 1] = itemName
		EasyLoot_NeedScrollBar_Update()
		EasyLootFilterItem:SetText("")
	end
	EasyLootFilterItem:ClearFocus()
end
